import mlflow
import os
import subprocess
import time

from multiprocessing import Process

# This listener writes *a lot* of metrics and may affect performance!
class PSThread(Process):
    def __init__(self, run_id, experiment_id=88):
        super(PSThread, self).__init__()
        self.run_id = run_id
        self.experiment_id = experiment_id
        self.parent_pid = os.getpid()

    def run(self):
        while True:
            output = (
                subprocess.run(
                    f"ps -p {self.parent_pid} -L -o pid,tid,psr,pcpu,%mem".split(),
                    capture_output=True,
                )
                .stdout.decode()
                .splitlines()
            )

            for line in output[1:]:
                line = line.strip().split(" ")
                line = [x for x in line if x.strip() != ""]

                pid = line[0]
                tid = line[1]
                psr = line[2]
                cpu = line[3]
                mem = line[4]

                mlflow.log_metric(f"PS - CPU {psr}", float(cpu))
                mlflow.log_metric(f"PS - MEM {psr}", float(mem))
            time.sleep(5)



import numpy as np

COLOURS = [31, 32, 34, 35, 36, 33]

CSV_FORMAT = np.dtype(
    [
        ("Experiment", int),
        ("Workload", int),
        ("Status", str),
        ("Run", str),
        ("Devices", str),
        ("Collocation", str),
        ("Listeners", str),
        ("File", str),
        ("Params", str),
    ]
)

COMMAND = (
    "mlflow run {Filepath} --env-manager={Envmanager} "
    "-P letter={Letter} "
    "-P workload={Workload} "
    "-P listeners={Listeners} "
    "-P file={File} "
    '-P params="-" '
)

RUN_LISTENERS = ["ps", "smi", "dcgmi", "top", "iostat"]

WORKLOAD_LISTENERS = {
    "nsys": "nsys profile --capture-range nvtx --nvtx-capture profile --cuda-memory-usage=true --capture-range-end repeat -o nsys_{Experiment}_{Workload}_{Letter} -f true -w true -x true -t cuda,nvtx ",
    "nsysraw": "nsys profile --cuda-memory-usage true -o nsys_{Experiment}_{Workload}_{Letter} -f true -w true -x true -t cuda,nvtx ",
    "ncu": "ncu -o ncu_{Experiment}_{Workload}_{Letter} -f --nvtx --nvtx-include profile ",
    "ncuraw": "ncu -o ncu_{Experiment}_{Workload}_{Letter} -f -c 100 ",
    "ncuattach": "ncu --mode=launch ",
}


MLPROJECT_CONTENTS = """name: radt

<REPLACE_ENV>

entry_points:
  main:
    parameters:
      letter: { type: string, default: ""}
      workload: { type: string, default: ""}
      listeners: { type: string, default: "smi+dcgmi+top" }
      params: { type: string, default: "-"}
      file: { type: string, default: "cifar10.py"}
      workload_listener: { type: string, default: ""}
    command: |
      <REPLACE_COMMAND>
"""

MLFLOW_COMMAND = (
    '''{WorkloadListener}python -m radt run -l {Listeners} -c {File} -p "{Params}"'''
)



import io
import mlflow
import subprocess

from datetime import datetime
from multiprocessing import Process

import os


class SMIThread(Process):
    def __init__(self, run_id, experiment_id=88):
        super(SMIThread, self).__init__()
        self.run_id = run_id
        self.experiment_id = experiment_id

    def run(self):
        SMI_GPU_ID = os.getenv("SMI_GPU_ID")

        print("SMI GPU ID:", SMI_GPU_ID)
        self.dcgm = subprocess.Popen(
            f"nvidia-smi -i {SMI_GPU_ID} -l 1 --query-gpu=power.draw,timestamp,utilization.gpu,utilization.memory,memory.used,pstate --format=csv,nounits,noheader".split(),
            stdout=subprocess.PIPE,
        )
        for line in io.TextIOWrapper(self.dcgm.stdout, encoding="utf-8"):
            line = line.split(", ")
            if len(line) > 1 and line[0] != "#":
                try:
                    m = {}
                    m["SMI - Power Draw"] = float(line[0])
                    m["SMI - Timestamp"] = datetime.strptime(
                        line[1] + "000", r"%Y/%m/%d %H:%M:%S.%f"
                    ).timestamp()
                    try:
                        m["SMI - GPU Util"] = float(line[2]) / 100
                    except ValueError:
                        m["SMI - GPU Util"] = float(-1)
                    try:
                        m["SMI - Mem Util"] = float(line[3]) / 100
                    except ValueError:
                        m["SMI - Mem Util"] = float(-1)
                    m["SMI - Mem Used"] = float(line[4])
                    m["SMI - Performance State"] = int(line[5][1:])
                    mlflow.log_metrics(m)
                except ValueError as e:
                    print("SMI Listener failed to report metrics")
                    break
